# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
import yaml
try:
    from gi import require_version
except ImportError:
    print("ERROR: pygobject is not installed.")
    raise
try:
    require_version("Gtk", "3.0")
except ValueError:
    print("ERROR: gtk3 development files are not installed.")
    raise
except AttributeError:
    print("pygobject version too old.")
    raise
from gi.repository import Gtk, Gdk
from gi.repository import GObject
from os.path import abspath
from os.path import dirname
from os.path import isfile
from os.path import join
from pkg_resources import resource_exists
from pkg_resources import resource_filename
from logging import getLogger
from logging import basicConfig
from logging import INFO
try:
    from synapselabels import __version__
    assert __version__
except ImportError:
    __version__ = "UNKNOWN"


log = getLogger('synapselabels')


label_config_file = 'labelconfig.yaml'


def get_qr_code(model, eth0mac, snap_mac, wifimac, meid, ccid, datecode):
    return (
        "#08:" +
        model + ":" +
        datecode + ":" +
        eth0mac[-6:] + ":" +
        snap_mac[-6:] + ":" +
        wifimac + ":" +
        meid + ":" +
        ccid + ":"
        ) if ccid else (
            "#02:" +
            model + ":" +
            datecode + ":" +
            eth0mac + ":" +
            snap_mac + ":" +
            wifimac +
            ((":" + meid) if meid else "")
            )


def create_labelFile_fcc(
        self, model, eth0mac, snap_mac, wifimac, meid, ccid, datecode):
    retval = ''
    # m - set measuring unit
    retval += "m i\n"
    # J - Job Start
    retval += "J\n"
    # S - Set Label Size
    retval += "S l1;0,0,1,1.125,2\n"
    # O - Set Print Options
    retval += "O R\n"
    # T - Text Field Definition
    if meid:
        if ccid:
            retval += "T 0.17,0.13,0,3,pt6;MODEL:\n"
            retval += "T 0.5,0.13,0,5,pt6;" + model + "\n"
            retval += "T 0.09,0.21,0,3,pt6;ETH MAC:\n"
            retval += "T 0.5,0.21,0,5,pt6;" + eth0mac + "\n"
            retval += "T 0.03,0.29,0,3,pt6;SNAP MAC:\n"
            retval += "T 0.5,0.29,0,5,pt6;" + snap_mac + "\n"
            retval += "T 0.08,0.37,0,3,pt6;WIFI MAC:\n"
            retval += "T 0.5,0.37,0,5,pt6;" + wifimac + "\n"
            retval += "T 0.06,0.45,0,3,pt6;MEID:\n"
            retval += "T 0.3,0.45,0,5,pt6;" + meid + "\n"
            retval += "T 0.06,0.53,0,3,pt6;CCID:\n"
            retval += "T 0.3,0.53,0,5,pt6;" + ccid + "\n"
            retval += "T 0.14,0.61,0,3,pt6;DC:\n"
            retval += "T 0.3,0.61,0,5,pt6;" + datecode + "\n"
        else:
            retval += "T 0.17,0.13,0,3,pt6;MODEL:\n"
            retval += "T 0.5,0.13,0,5,pt6;" + model + "\n"
            retval += "T 0.09,0.22,0,3,pt6;ETH MAC:\n"
            retval += "T 0.5,0.22,0,5,pt6;" + eth0mac + "\n"
            retval += "T 0.03,0.31,0,3,pt6;SNAP MAC:\n"
            retval += "T 0.5,0.31,0,5,pt6;" + snap_mac + "\n"
            retval += "T 0.08,0.4,0,3,pt6;WIFI MAC:\n"
            retval += "T 0.5,0.4,0,5,pt6;" + wifimac + "\n"
            retval += "T 0.26,0.49,0,3,pt6;MEID:\n"
            retval += "T 0.5,0.49,0,5,pt6;" + meid + "\n"
            retval += "T 0.34,0.58,0,3,pt6;DC:\n"
            retval += "T 0.5,0.58,0,5,pt6;" + datecode + "\n"
    else:
        retval += "T 0.17,0.13,0,3,pt6;MODEL:\n"
        retval += "T 0.5,0.13,0,5,pt6;" + model + "\n"
        retval += "T 0.09,0.23,0,3,pt6;ETH MAC:\n"
        retval += "T 0.5,0.23,0,5,pt6;" + eth0mac + "\n"
        retval += "T 0.03,0.33,0,3,pt6;SNAP MAC:\n"
        retval += "T 0.5,0.33,0,5,pt6;" + snap_mac + "\n"
        retval += "T 0.08,0.43,0,3,pt6;WIFI MAC:\n"
        retval += "T 0.5,0.43,0,5,pt6;" + wifimac + "\n"
        retval += "T 0.34,0.53,0,3,pt6;DC:\n"
        retval += "T 0.5,0.53,0,5,pt6;" + datecode + "\n"
    qrcode = self.get_qr_code(
        model, eth0mac, snap_mac, wifimac, meid, ccid, datecode)
    # TBD automate this better
    if len(qrcode) >= 80:
        retval += "B 1.39,0.08,0,QRCODE+ELQ+MODEL2,.0133333;" + qrcode + "\n"
    else:
        retval += "B 1.39,0.08,0,QRCODE+ELH+MODEL2,.0133333;" + qrcode + "\n"
    retval += "T 0.07,0.70,0,5,pt4.5;" +\
        "This device complies with part 15 of the FCC Rules.\n"
    retval += "T 0.07,0.76,0,5,pt4.5;" +\
        "Operation is subject to the following two conditions:\n"
    retval += "T 0.07,0.82,0,5,pt4.5;" +\
        "1) This device may not cause harmful interference, and\n"
    retval += "T 0.07,0.88,0,5,pt4.5;" +\
        "2) This device must accept any interference received,\n"
    retval += "T 0.07,0.94,0,5,pt4.5;" +\
        "    including interference that may cause undesired operation.\n"
    retval += "G 0.05,0.64,0;R:1.9,0.33,0.0066666\n"
    retval += "A 1\n"
    return retval


def set_treeview_columns(treeview, columns):
    for i in range(len(columns)):
        # cellrenderer to render the text
        cell = Gtk.CellRendererText(editable=True)
        # cell.set_property("editable", True)
        # the text in the first column should be in boldface
        # if i == 0:
        #    cell.props.weight_set = True
        #    cell.props.weight = Pango.Weight.BOLD
        # the column is created
        col = Gtk.TreeViewColumn(
            columns[i], cell, text=i)
        # and it is appended to the treeview
        treeview.append_column(col)


def set_treeview_from_clipboard(treeview, with_header=True):
    textData = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD).wait_for_text()
    listmodel = None
    if textData is not None:
        print("clipboard text:", repr(textData))
        for line in textData.splitlines():
            # print("line:", repr(line))
            texts = line.replace(',', '\t').split('\t')
            if listmodel is None:
                for column in treeview.get_columns():
                    treeview.remove_column(column)
                args = [GObject.TYPE_STRING] * len(texts)
                print(args)
                listmodel = Gtk.ListStore(*args)
                # listmodel = Gtk.ListStore(str, str, str)
                if with_header:
                    columns = texts
                else:
                    columns = [
                        "Column{}".format(index) for
                        index in
                        range(0, len(texts))
                        ]
                    print("appending:", texts)
                    listmodel.append(texts)
            else:
                print("appending:", texts)
                listmodel.append(texts)
        if listmodel is not None:
            print("setting model:", listmodel)
            treeview.set_model(listmodel)
            set_treeview_columns(treeview, columns)
    else:
        print("No text on the clipboard.")


def get_path(name):
    """
    Returns the absolute path name of a resource file
    File in the current directory overrides file in package
    """
    path = None
    if isfile(name):
        path = abspath(name)
    elif resource_exists(__name__, name):
        res_filename = resource_filename(__name__, name)
        res_path = abspath(join(dirname(__file__), res_filename))
        if isfile(res_path):
            path = res_path
    if path:
        log.info("located path: %s", path)
    else:
        log.error("could not locate path for: %s", name)
    return path


def fill_combo(combo, data):
    store = Gtk.ListStore(GObject.TYPE_STRING)
    for line in data:
        store.append([line])
    combo.set_model(store)
    combo.set_active(0)
    # And here's the new stuff:
    cell = Gtk.CellRendererText()
    combo.pack_start(cell, True)
    combo.add_attribute(cell, "text", 0)


class LabelsGui(object):
    """
    This is the E20 GUI
    """
    def __init__(self):
        """
        Build GUI
        """
        # Build GUI from Glade file
        self.builder = Gtk.Builder()
        self.builder.add_from_file(get_path('labelsgui.glade'))
        # Get objects
        go = self.builder.get_object
        self.window = go('applicationwindow1')
        self.window.set_title('Synapse Labels - v{}'.format(__version__))
        self.treeview_data_table = go('treeviewDataTable')
        self.treeview_data_table.get_selection().connect(
            "changed",
            self.tree_selection_changed,
            "dummy"
            )
        # self.combobox = go('comboboxtext1')
        # self.mac_num = go('labelMacNum')
        self.buttonPaste = go('buttonPaste')
        self.buttonPaste.connect(
            "clicked",
            self.button_paste_clicked,
            "cool button 1"
            )
        self.buttonPasteWithHeader = go('buttonPasteWithHeader')
        self.buttonPasteWithHeader.connect(
            "clicked",
            self.button_paste_with_header_clicked,
            "cool button 2"
            )
        self.buttonPrintSelectedLabel = go('buttonPrintSelectedLabel')
        self.buttonPrintSelectedLabel.connect(
            "clicked",
            self.print_selected_label,
            )
        self.cb_printers = go('comboboxPrinters')
        self.cb_models = go('comboboxModels')
        self.process_config()
        # Connect signals
        self.builder.connect_signals(self)
        # Configure interface
        self.window.connect('delete-event', lambda x, y: Gtk.main_quit())
        # GObject.timeout_add(1000, self.update_mac_pool)
        # Everything is ready
        self.window.show()

    def print_selected_label(self, widget, data=None):
        print("print_selected_label")
        (model, pathlist) = self.treeview_data_table.get_selection(
            ).get_selected_rows()
        for path in pathlist:
            tree_iter = model.get_iter(path)
            number_of_columns = model.get_n_columns()
            print('number_of_columns:', number_of_columns)
            data = {}
            for index, column in enumerate(
                    self.treeview_data_table.get_columns()):
                data[column.get_title()] = model.get_value(tree_iter, index)
            print(data)

    def process_config(self):
        with open(label_config_file, 'r') as stream:
            yaml_data = yaml.load(stream)
        print('yaml_data:', repr(yaml_data))
        if 'label types' in yaml_data:
            self.label_types = yaml_data['label types']
            print('label types:', repr(self.label_types))
        else:
            raise ValueError('"label types" is not in config file')
        if 'printers' in yaml_data:
            printers = yaml_data['printers']
            print('printers:', repr(printers))
            fill_combo(self.cb_printers, printers.keys())
        else:
            raise ValueError('"printers" is not in config file')
        if 'models' in yaml_data:
            models = yaml_data['models']
            print('models:', repr(models))
            fill_combo(self.cb_models, models)
        else:
            raise ValueError('"models" is not in config file')

    def button_paste_clicked(self, widget, data=None):
        print("button_paste_clicked - %s" % data)
        set_treeview_from_clipboard(self.treeview_data_table, with_header=False)

    def button_paste_with_header_clicked(self, widget, data=None):
        print("button_paste_with_header_clicked - %s" % data)
        set_treeview_from_clipboard(self.treeview_data_table, with_header=True)

    def tree_selection_changed(self, widget, data=None):
        # print("selection_changed - %s" % data)
        csr = widget.count_selected_rows()
        # print("count_selected_rows:", csr)
        self.buttonPrintSelectedLabel.set_sensitive(csr > 0)


def synapse_labels():
    basicConfig(level=INFO)
    gui = LabelsGui()
    assert gui
    Gtk.main()


if __name__ == "__main__":
    synapse_labels()
